use crate::admin::route::Route;
use axum::Router;
use chrono::NaiveDateTime;
use serde::{Deserialize, Serialize};
use sqlx::{FromRow, SqlitePool};
use std::sync::Arc;

mod web;

#[derive(FromRow, Debug, Serialize, Deserialize)]
pub(crate) struct Station {
    pub(crate) id: i64,
    pub(crate) name: String,
    notes: Option<String>,
    amount_people: Option<i64>,
    last_login: Option<NaiveDateTime>, // TODO use proper timestamp (NaiveDateTime?)
    pw: String,
    lat: Option<f64>,
    lng: Option<f64>,
}

impl Station {
    pub(crate) async fn all(db: &SqlitePool) -> Vec<Self> {
        sqlx::query_as::<_, Self>(
            "SELECT id, name, notes, amount_people, last_login, pw, lat, lng FROM station;",
        )
        .fetch_all(db)
        .await
        .unwrap()
    }

    pub async fn find_by_id(db: &SqlitePool, id: i64) -> Option<Self> {
        sqlx::query_as!(
            Self,
            "SELECT id, name, notes, amount_people, last_login, pw, lat, lng FROM station WHERE id = ?",
            id
        )
        .fetch_one(db)
        .await
        .ok()
    }

    async fn create(db: &SqlitePool, name: &str) -> Result<(), String> {
        sqlx::query!("INSERT INTO station(name) VALUES (?)", name)
            .execute(db)
            .await
            .map_err(|e| e.to_string())?;
        Ok(())
    }

    async fn update_name(&self, db: &SqlitePool, name: &str) {
        sqlx::query!("UPDATE station SET name = ? WHERE id = ?", name, self.id)
            .execute(db)
            .await
            .unwrap();
    }

    async fn update_notes(&self, db: &SqlitePool, notes: &str) {
        sqlx::query!("UPDATE station SET notes = ? WHERE id = ?", notes, self.id)
            .execute(db)
            .await
            .unwrap();
    }

    async fn update_amount_people(&self, db: &SqlitePool, amount_people: i64) {
        sqlx::query!(
            "UPDATE station SET amount_people = ? WHERE id = ?",
            amount_people,
            self.id
        )
        .execute(db)
        .await
        .unwrap();
    }

    async fn update_amount_people_reset(&self, db: &SqlitePool) {
        sqlx::query!(
            "UPDATE station SET amount_people = NULL WHERE id = ?",
            self.id
        )
        .execute(db)
        .await
        .unwrap();
    }

    async fn update_location(&self, db: &SqlitePool, lat: f64, lng: f64) {
        sqlx::query!(
            "UPDATE station SET lat = ?, lng = ? WHERE id = ?",
            lat,
            lng,
            self.id
        )
        .execute(db)
        .await
        .unwrap();
    }
    async fn update_location_clear(&self, db: &SqlitePool) {
        sqlx::query!(
            "UPDATE station SET lat = NULL, lng=NULL WHERE id = ?",
            self.id
        )
        .execute(db)
        .await
        .unwrap();
    }

    async fn delete(&self, db: &SqlitePool) -> Result<(), String> {
        sqlx::query!("DELETE FROM station WHERE id = ?", self.id)
            .execute(db)
            .await
            .map_err(|e| e.to_string())?;
        Ok(())
    }

    async fn routes(&self, db: &SqlitePool) -> Vec<Route> {
        sqlx::query_as::<_, Route>(
            "
            SELECT r.id, r.name
            FROM route r
            JOIN route_station rs ON r.id = rs.route_id
            WHERE rs.station_id = ?
            ORDER BY rs.pos
            ",
        )
        .bind(self.id)
        .fetch_all(db)
        .await
        .unwrap()
    }

    pub async fn is_in_route(&self, db: &SqlitePool, route: &Route) -> bool {
        for r in self.routes(db).await {
            if r.id == route.id {
                return true;
            }
        }

        false
    }
}

pub(super) fn routes() -> Router<Arc<SqlitePool>> {
    web::routes()
}
